/*------------------------------------------------------- File Description -*\

  Copyright (C) Odin TeleSystems Inc., 1994-1996

  NAME

    BLD-DEMO.C

  DESCRIPTION

  Example of using the balder board in generation mode.

\*--------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>  //
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <time.h>

#include "balder.h"              /* Balder Standard Application Programming Interface */
#ifdef BLD_UTILS
#include "bldutils.h"            /* Balder Utils Application Programming Interface */
#endif

typedef enum {
  FALSE=0,
  TRUE=1
} Bool;

#define EXIT_CODE_OK    0              /* Exit code from main                */
#define EXIT_CODE_ERROR 1              /* Exit code from main                */

//#define BALDER_BOARD_TYPE  102         /* Unique code for Balder-2           */
#define BALDER_BOARD_TYPE  108         /* Unique code for Balder-8           */

#define NO_OF_BOARDS  1                /* We can have up to 8 boards         */
static int irqTable[NO_OF_BOARDS] = { 0 };

#if BALDER_BOARD_TYPE == 102

# define  LI_MAX (NO_OF_BOARDS * 2 - 1)

#elif BALDER_BOARD_TYPE == 108

# define LI_MAX (NO_OF_BOARDS * 8 - 1) /* Two LIs per board (0 and 1) on     */
                                       /* Balder-2, eight on Balder-8        */
// # define LI_MAX  3
#endif

BldRc layer1Status[LI_MAX + 1];

static int addressTable[] = {
  0x000,
  0x200,
  0x220,
  0x240,
  0x260,
  0x280,
  0x2A0,
  0x2C0,
  0x2E0,
  0x300,
  0x320,
  0x340,
  0x360,
  0x380
};


static FILE *logF = NULL;
#define DEFAULT_LOG_FILE "bld-demo.log"

static void idleFunction(void);

static Bool installDriver(int BALDER_BASE_ADDR, int *irqTable);
static void uninstallDriver(void);
static int getBaseAddress(void);
static int getIRQ(void);
static void resetHardware(void);
static void logf(char *formatStr, ...);


/*--------------------------------------------------------------------------*\
  DESCRIPTION:

    An example of how to use the balder.h API with one Balder-2 board
    configured for generation (all LIs in TE mode)
\*--------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
  int BALDER_BASE_ADDR = 0x00;
  int currLiNo = 0; // Used for transmitting
  int currBchan = 1;
  int i;
  BldRc rc, rc1, rc2;
  int           bChannel;
  unsigned char byte;
  unsigned char *fmBuf; // Used for transmitting
  int bigLen = 0;
  int howMany = 1;
  int exitFlag = FALSE;
  static unsigned char bigBuf[300];
  int c;
  int continuous = FALSE;
  char bNumber[28+1];
  int idleEnabled = TRUE;
  char *logFileName;

  printf("BLD-DEMO, %s, Copyright (c) Odin TeleSystems Inc. 1994\n", __DATE__);
  printf("Using Driver: %s\n\n", bldIdentDriver());

  if (argc == 1) {
    logFileName = DEFAULT_LOG_FILE;
  } else {
    logFileName = argv[1];
  }

  if ((logF = fopen(logFileName, "wt")) == NULL) {
    printf("Couldn't open logfile %s\n", logFileName);
    exit(-1);
  }

#if 1 // Make this zero if you get tired of entering the address and irq every time
  BALDER_BASE_ADDR = getBaseAddress();
  irqTable[0] = getIRQ();
#else
  BALDER_BASE_ADDR = 0x280;
  irqTable[0] = 10;
#endif

  if (!installDriver(BALDER_BASE_ADDR, irqTable)) {
    return EXIT_CODE_ERROR;
  }

  logf("Press '?' for help, 'q' to quit\n");

  // Main loop
  while (!exitFlag) {

    if (idleEnabled) {
      idleFunction();  // Read from D-channels
    }

    if (continuous) {
      for (i = 0; i < 100; i++) {
        fmBuf = (unsigned char *) "\x02\xFF\x03\x08\x01\x77\x05\x04\x03\x80\x90\xA2\x18\x01\x82\x34\x01\x40\x3B\x02\x81\x80\x96\x7B\x01\x83";
        if ((rc = bldWriteLi(currLiNo, fmBuf, 26)) != BLD_SUCCESS) {
          logf("Error #%d: %s\n", (int) rc, bldGetErrMsg(rc));
        } else {
          //logf("Li %d: Sending Setup, 02 FF 03 08 01 77 05 04 03 80 90 A2 18 01 82 34 01 40 3B 02 81 80 96 7B 01 83   \n", currLiNo);
        }
      }
    }

    if (kbhit()) {
      switch (c = getch()) {

      case '?':
      case 'h':
        logf("Cmd  Synopsis\n");
        logf("===  ========\n");
        logf(" ?   Help text\n");
        logf(" h   Help text\n");
        logf(" 0-7 Select access, this access is then used for subsequent commands\n");
        logf(" a   Activate Li\n");
        logf(" b   Read B-channel byte from Li (and Bch)\n");
        logf(" d   Deactivate Li\n");
        logf(" g   Generate messages continuously\n");
        logf(" j   Blast messages on all 8 accesses\n");
        logf(" k   Kill continuous generation\n");
        logf(" l   Loop Bchannel seen from interface side\n");
        logf(" q   Quit the program\n");
        logf(" r   Reset all Hw and layer 1 on the board\n");
        logf(" s   Display layer1 status\n");
        logf(" t   Send TEI request: FC FF 03 0F 29 6B 01 FF\n");
        logf(" v   Sending a long nonsense message\n");
        logf(" w   Sending two long nonsense messages back-to-back\n");
        logf(" y   Send Setup message\n");
        logf(" !   Send continuous bytes on a B-channel\n");
        logf(" #   Turn byte sending off for a B-channel\n");
        logf(" +   Enter total msg Length to be transmitted when 'v' or 'w' is pressed:\n");
        logf(" -   Enter how many messages to be sent when 'v' is pressed\n");
        logf(" @   Enable test mode, only a reset disables\n");
        logf(" {   Connect handset\n");
        logf(" }   Disconnect handset\n");
        logf(" |   Disconnect terminating resistor\n");
        break;

      case '<':
        currBchan = 1;
        logf("Current B-channel: %d\n", currBchan);
        break;

      case '>':
        currBchan = 2;
        logf("Current B-channel: %d\n", currBchan);
        break;

      case 'k':
        logf("Li %d: Kill continuous generation\n", currLiNo);
        continuous = FALSE;
        break;


#ifdef BLD_UTILS
      case 'l':
        /* Set Byte on an a B-channel */
        logf("Li %d: Loop Bchannel.\n", currLiNo);
        logf("B-channel: ");
        scanf("%d", &bChannel);
        logf("Li %d: Looping b-channel %d\n", currLiNo, bChannel);
        if ((rc = bldBchLoopOn(currLiNo, bChannel)) != BLD_SUCCESS) {
          logf("Error: %s\n", bldGetErrMsg(rc));
        }
        break;
#endif

#ifdef BLD_UTILS
      case '@':
        logf("Li %d: Entering test mode, only a reset disables\n", currLiNo);
        bldTestMode(currLiNo);
        break;
#endif

      case 'q':
        /* Quit program */
        exitFlag = TRUE;
        break;

      case 'a':
        /* Layer 1 Activate Request */
        logf("Li %d: Activating\n", currLiNo);
        if ((rc = bldActivate(currLiNo)) != BLD_SUCCESS) {
          logf("Error: %s\n", bldGetErrMsg(rc));
        }
        break;

#if 0
      case 'j':
        logf("Message blasting on 8 accesses. Press any key to stop\n");
        messageBlasting();
        break;
#endif


      case 'i':
        logf("Idle function disabled\n");
        idleEnabled = FALSE;
        break;

      case 'I':
        logf("Idle function enabled\n");
        idleEnabled = TRUE;
        break;

      case 'd':
        /* Layer 1 Deactivate Request */
        logf("Li %d: Deactivating\n", currLiNo);
        bldDeactivate(currLiNo);
        break;

      case 'b':
        /* Read a byte from the two B-channels and print it */
        for (i= 1; i <= 2; i++) {
          rc = bldByteReadBch(currLiNo, i, &byte); // Read a byte from current Li's B-channel i
          switch (rc) {
          case BLD_SUCCESS:
            logf("Li %d: B-channel %d byte: %02X\n", currLiNo, i, (int) byte);
            break;
          case BLD_TIMEOUT:
            logf("Timeout. Could not read a B-channel %d byte\n", i);
            break;
          default:
            logf("Internal Error: Unknown return code form bldByteReadBch()\n");
            break;
          }
        }
        break;

      case '!':
        /* Set Byte on an a B-channel */
        logf("Li %d: Set Byte on an a B-channel.\n", currLiNo);
        logf("Byte: ");
        scanf("%X", &byte);
        logf("B-channel: ");
        scanf("%d", &bChannel);
        logf("Li %d: Writing 0x%.2X on b-channel %d\n", currLiNo, byte, bChannel);
        if ((rc = bldByteOnBch(currLiNo, bChannel, byte)) != BLD_SUCCESS) {
          logf("Error: %s\n", bldGetErrMsg(rc));
        }
        break;

      case '#':
        /* Set Byte off a B-channel */
        logf("Li %d: Turn Byte off for a B-channel.\n", currLiNo);
        logf("B-channel: ");
        scanf("%d", &bChannel);
        logf("OK, turning off byte for b-channel %d\n", bChannel);
        if ((rc = bldByteOffBch(currLiNo, bChannel)) != BLD_SUCCESS) {
          logf("Error: %s\n", bldGetErrMsg(rc));
        }
        break;

      case '{':
        /* Connect handset */
        logf("Li %d: Connect handset. b-channel %d\n", currLiNo, currBchan);
        if ((rc = bldPhoneOn(currLiNo, currBchan)) != BLD_SUCCESS) {
          logf("Error: %s\n", bldGetErrMsg(rc));
        }
        break;

      case '}':
        /* Disconnect handset */
        logf("Li %d: Disconnect handset. b-channel %d\n", currLiNo, currBchan);
        if ((rc = bldPhoneOff(currLiNo, currBchan)) != BLD_SUCCESS) {
          logf("Error: %s\n", bldGetErrMsg(rc));
        }
        break;


      case '$':
        logf("Li %d: Reseting Arcofi %d\n", currLiNo);
        if ((rc = bldResetCodec(currLiNo)) != BLD_SUCCESS) {
          logf("Error: %s\n", bldGetErrMsg(rc));
        }
        break;

      case '^':
        /* Dial DTMF mode */
        logf("Dial mode\n");
        logf("B-number: ");
        scanf("%s", bNumber);
        logf("Li %d: Calling %s on B-channel %d\n", currLiNo, bNumber, currBchan);
        if ((rc = bldDtmfOnBch(currLiNo, currBchan, bNumber, 6, 2)) != BLD_SUCCESS) {
          logf("Error: %s\n", bldGetErrMsg(rc));
        }
        break;

      case '%':
        /* Set DTMF Tone on an a B-channel */
        logf("Li %d: Set DTMF Tone on an a B-channel.\n", currLiNo);
        logf("B-channel: ");
        scanf("%d", &bChannel);
        logf("Li %d: Tone on b-channel %d\n", currLiNo, bChannel);
        if ((rc = bldDtmfOnBch(currLiNo, bChannel, "#", 0, 0)) != BLD_SUCCESS) {
          logf("Error: %s\n", bldGetErrMsg(rc));
        }
        break;

#ifdef DEBUG_TONE
      case '(':
        /* Set Tone on an a B-channel */
        logf("Li %d: Set Tone on an a B-channel.\n", currLiNo);
        logf("B-channel: ");
        scanf("%d", &bChannel);
        logf("Li %d: Tone on b-channel %d\n", currLiNo, bChannel);
        if ((rc = bldToneOnBch(currLiNo, bChannel)) != BLD_SUCCESS) {
          logf("Error: %s\n", bldGetErrMsg(rc));
        }
        break;

      case ')':
        /* Set Tone off a B-channel */
        logf("Li %d: Turn Tone off for a B-channel.\n", currLiNo);
        logf("B-channel: ");
        scanf("%d", &bChannel);
        logf("OK, turning off tone for b-channel %d\n", bChannel);
        if ((rc = bldToneOffBch(currLiNo, bChannel)) != BLD_SUCCESS) {
          logf("Error: %s\n", bldGetErrMsg(rc));
        }
        break;
#endif

      case 's':
        /* Check status Li */
        rc = bldGetStatusLi(currLiNo);
        logf("Li %d: Layer1 status = %s\n", currLiNo, (rc == BLD_L1_DOWN) ? "Down" : "OK");
        break;

      case 'r':
        /* Reset all HDLCs */
        logf("Resetting All HDLCs on the board (and Layer 1)\n");
        resetHardware();
        break;

      case '-':
        /* Enter how many messages to be sent when 'v' is pressed */
        logf("Enter how many msgs that should be transmitted when 'v' is pressed: ");
        scanf("%d", &howMany);
        break;

      case '+':
        /* Enter a length for the generated message */
        logf("Enter total msg Length to be transmitted when 'v' is pressed: ");
        scanf("%d", &bigLen);
        break;

      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
        /* Select LI for sending of the generated message */
        logf("Li %d: Selected\n", (int) c - '0');
        currLiNo = c - '0';
        break;

      case 'R':
        /* Chip reset an Li Pair */
        logf("Li%d, Li%d: Chip reset an Li Pair \n", currLiNo, currLiNo + 1);
        if ((rc = bldChipResetLiPair(currLiNo, currLiNo + 1)) != BLD_SUCCESS) {
          logf("Error #%d: %s\n", (int) rc, bldGetErrMsg(rc));
        }
        break;


      case 't':
        /* Send a TEI message */
        fmBuf = (unsigned char *) "\xFC\xFF\x03\x0F\x29\x6B\x01\xFF";
        if ((rc = bldWriteLi(currLiNo, fmBuf, 8)) != BLD_SUCCESS) {
          logf("Error #%d: %s\n", (int) rc, bldGetErrMsg(rc));
        } else {
          logf("Li %d: Sending TEI request:  FC FF 03 0F 29 6B 01 FF  \n", currLiNo);
        }
        break;


      case 'y':
        /* Send a longer message */
        fmBuf = (unsigned char *) "\x02\x85\x00\x00\x08\x01\x77\x05\x04\x03\x80\x90\xA2\x18\x01\x82\x34\x01\x40\x3B\x02\x81\x80\x96\x7B\x01\x83";
        if ((rc = bldWriteLi(currLiNo, fmBuf, 27)) != BLD_SUCCESS) {
          logf("Error #%d: %s\n", (int) rc, bldGetErrMsg(rc));
        } else {
          logf("Li %d: Sending Setup, 02 85 00 00 08 01 77 05 04 03 80 90 A2 18 01 82 34 01 40 3B 02 81 80 96 7B 01 83   \n", currLiNo);
        }
        break;

      case 'v':
        /* Send a long nonsense message ('+') */
        memcpy(bigBuf, (unsigned char *) "\x02\xFF\x03\x08\x01\x77\x05", 7);

        for (i = 7; i < bigLen; i++) {
          bigBuf[i] = (unsigned char) ((i + 1) % 10);
        }

        logf("Li %d: Sending %d long nonsense message:\n", currLiNo, howMany);
        for (i = 0; i < bigLen; i++) {
          logf("%02X ", (int) bigBuf[i]);
        }
        logf("\n");

        for (i = 0; i < howMany; i ++) {
          if ((rc = bldWriteLi(currLiNo, bigBuf, bigLen)) != BLD_SUCCESS) {
            logf("Error #%d: %s\n", (int) rc, bldGetErrMsg(rc));
          }
        }
        break;

      case 'w':
        /* Send two back-to-back long nonsense message ('+') */
        memcpy(bigBuf, (unsigned char *) "\x02\xFF\x03\x08\x01\x77\x05", 7);

        for (i = 7; i < bigLen; i++) {
          bigBuf[i] = (unsigned char) ((i + 1) % 10);
        }

        logf("Li %d: Sending two back-to-back long nonsense message:\n", currLiNo);
        for (i = 0; i < bigLen; i++) {
          logf("%02X ", (int) bigBuf[i]);
        }
        logf("\n");

        rc1 = bldWriteLi(currLiNo, bigBuf, bigLen);
        rc2 = bldWriteLi(currLiNo, bigBuf, bigLen);

        if (rc1 != BLD_SUCCESS) {
          logf("Error #%d: %s\n", (int) rc, bldGetErrMsg(rc));
        }
        if (rc2 != BLD_SUCCESS) {
          logf("2 Error #%d: %s\n", (int) rc, bldGetErrMsg(rc));
        }
        break;

#ifdef BLD_UTILS
      case '|':
        logf("Li %d: Disconnecting Terminating Resistor\n", currLiNo);
        bldTermLi(currLiNo, FALSE);
        break;
#endif


      default:
        logf("Unknown command. Press '?' for help, 'q' to quit\n");
        break;
      }
    }
  }

  logf("Normal program termination...\n");

  uninstallDriver();

  return EXIT_CODE_OK;

}

#if 0
/*--------------------------------------------------------------------------*\

  DESCRIPTION:

    Blasts messages an all 8 accesses.


\*--------------------------------------------------------------------------*/
static void messageBlasting()        // funcdef
{
  #define  MODULO_INCREMENT(qi,qs)     (qi++, qi %= qs);
  unsigned char *p;
  unsigned short i;
  int liNo;
  int txLi = 0;
  Byte *txBuf;
  Byte *rxBuf;
  #define TX_BUF_SIZE 150
  #define RX_BUF_SIZE TX_BUF_SIZE + 10
  BldFrameHeader fmHeader;
  BldRc bldRc;
  int parAcs = 8;  // No of accessess in parallell
  Ulong busyCount[

  if ((txBuf = (Byte *)malloc(TX_BUF_SIZE)) == NULL) {
    logf("Error: Could not allocate tx buffer \n");
    uninstallDriver(); exit(-1);
  }

  if ((rxBuf = (Byte *)malloc(RX_BUF_SIZE)) == NULL) {
    logf("Error: Could not allocate tx buffer \n");
    uninstallDriver(); exit(-1);
  }

  while (!kbhit()) {

    if ((bldRc = bldWriteLi(txLi, fmBuf, 26)) == BLD_SUCCESS) {
      pendingMsg[txLi] += parAcs;
      MODULO_INCREMENT(txLi, parAcs);
    } else if (bldRc == BLD_TX_BUSY) {
      if (
    }
      logf("Error #%d: %s\n", (int) rc, bldGetErrMsg(rc));

  }
  getch();
  if (bldRead(rxBuf, BUF_SIZE, &fmHeader) != BLD_NO_FRAMES) {

    logf(">> Li:%d ", (int) fmHeader.liNo);
    logf("#%03d " , (int) fmHeader.fmSeqNo);
    logf("%02d:"  , (int) fmHeader.hour);
    logf("%02d:"  , (int) fmHeader.min);
    logf("%02d."  , (int) fmHeader.sec);
    logf("%02d [" , (int) fmHeader.csec);
    logf("%.2X] " , (int) fmHeader.fmStatus);

    p = rxBuf;

    switch (fmHeader.fmType) {
    case BLD_FM_I: // Only used on auto-mode
      logf("I-frame ");
      for (i = 0; i < fmHeader.fmLength; i++) {
        logf("%02X ", (int) *p++);
      }
      break;
    case BLD_FM_UI: // Only used in auto-mode
      logf("UI-frame ");
      for (i = 0; i < fmHeader.fmLength; i++) {
        logf("%02X ", (int) *p++);
      }
      break;
    case BLD_FM_HDLC:
      for (i = 0; i < fmHeader.fmLength; i++) {
        logf("%02X ", (int) *p++);
      }
      break;
    case BLD_FM_STR: // '\0' terminated info string
      logf("%s", rxBuf);
      break;
    default:
      logf("Error: Invalid frame type");
      break;
    }
    logf("\n");;
  } else {
    // Don't print this stuff until we have encountered an empty read buffer

    /* Check layer 1 status, print any change (Although deactivation and  */
    /* Activations are endicated by STR messages returned by bldRead()    */
    for (liNo = 0; liNo <= LI_MAX ; liNo++) {
      BldRc rc;
      rc = bldGetStatusLi(liNo);
      if (layer1Status[liNo] != rc) {
        logf("Li:%d Layer 1 status = %s\n", liNo, (rc == BLD_L1_DOWN) ? "Down" : "OK");
        layer1Status[liNo] = rc;
      }
    }
  }
}
#endif


/*--------------------------------------------------------------------------*\
  DESCRIPTION:
    Reads a message from any LI (if a message is available).
    If more than one message is available the oldest message
    is read.
\*--------------------------------------------------------------------------*/
static void idleFunction()        // funcdef
{
  unsigned char *p;
  unsigned short i;
  int liNo;

  #define BUF_SIZE  270    /* Must be at least 270 to accomodate all  */
                           /* .. possible (legal) ISDN messages       */
  static unsigned char msgBuf[BUF_SIZE];
  BldFrameHeader fmHeader;

  if (bldRead(msgBuf, BUF_SIZE, &fmHeader) != BLD_NO_FRAMES) {

    logf(">> Li:%d ", (int) fmHeader.liNo);
    logf("#%03d " , (int) fmHeader.fmSeqNo);
    logf("%02d:"  , (int) fmHeader.hour);
    logf("%02d:"  , (int) fmHeader.min);
    logf("%02d."  , (int) fmHeader.sec);
    logf("%02d [" , (int) fmHeader.csec);
    logf("%.2X] " , (int) fmHeader.fmStatus);

    p = msgBuf;

    switch (fmHeader.fmType) {
    case BLD_FM_I: // Only used on auto-mode
      logf("I-frame ");
      for (i = 0; i < fmHeader.fmLength; i++) {
        logf("%02X ", (int) *p++);
      }
      break;
    case BLD_FM_UI: // Only used in auto-mode
      logf("UI-frame ");
      for (i = 0; i < fmHeader.fmLength; i++) {
        logf("%02X ", (int) *p++);
      }
      break;
    case BLD_FM_HDLC:
      for (i = 0; i < fmHeader.fmLength; i++) {
        logf("%02X ", (int) *p++);
      }
      break;
    case BLD_FM_STR: // '\0' terminated info string
      logf("%s", msgBuf);
      break;
    default:
      logf("Error: Invalid frame type");
      break;
    }
    logf("\n");;
  } else {
    // Don't print this stuff until we have encountered an empty read buffer

    /* Check layer 1 status, print any change (Although deactivation and  */
    /* Activations are endicated by STR messages returned by bldRead()    */
    for (liNo = 0; liNo <= LI_MAX ; liNo++) {
      BldRc rc;
      rc = bldGetStatusLi(liNo);
      if (layer1Status[liNo] != rc) {
        logf("Li:%d Layer 1 status = %s\n", liNo, (rc == BLD_L1_DOWN) ? "Down" : "OK");
        layer1Status[liNo] = rc;
      }
    }
  }
}


/*---------------------------------------------------- Function Description -*\
  DESCRIPTION
    Sets up the driver by initializing the software with the
    proper IRQs and such, and then all LI are configured.

  RETURNS
    TRUE   // No error(s)
    FALSE  // Error(s)
\*---------------------------------------------------------------------------*/
static Bool installDriver(int BALDER_BASE_ADDR, int *irqTable)          // funcdef
{
  BldRc rc; // Return code from BLD functions
  int liNo;

  if ((rc = bldConstructDriver(BALDER_BOARD_TYPE, BALDER_BASE_ADDR, NO_OF_BOARDS, irqTable)) != BLD_SUCCESS) {
    logf("Error #%d: %s\n", (int) rc, bldGetErrMsg(rc));
    return FALSE;      /* Error, No reason to continue */
  }

  for (liNo=0; liNo <= LI_MAX; liNo++) {
    /* Configure a line interface.                         */
    /* for generation we always configure all LIs as TEs   */

#ifdef BLD_UTILS
    bldTermLi(liNo, TRUE); // Connect terminating resistor
#endif

    if ((rc = bldConfigureLi(liNo, BLD_TE)) != BLD_SUCCESS) {
      if (rc == BLD_L1_DOWN) {
        logf("Li:%d  Configured OK, but Layer 1 is inactive\n", liNo);
        layer1Status[liNo] = BLD_L1_DOWN; /* Stops idleFunction from
                                          .. printing it again            */
      } else {
        logf("Li:%d  Error #%d: %s\n", liNo, (int) rc, bldGetErrMsg(rc));
        return FALSE;  /* Error, No reason to continue */
      }
    }
  }

  return TRUE;  // No Errors but L1 may be down, who knows...
}

/*---------------------------------------------------- Function Description -*\
  DESCRIPTION
    Uninstall the driver.
\*---------------------------------------------------------------------------*/
static void uninstallDriver()       // funcdef
{
  BldRc rc;
  if ((rc = bldDestructDriver()) != BLD_SUCCESS) {
    logf("Error #%d: %s\n", (int) rc, bldGetErrMsg(rc));
  }
}

#if 0
/*---------------------------------------------------- Function Description -*\
  DESCRIPTION
    Resets the software part of the driver. Basically initializes all
    buffers and data in the driver.
\*---------------------------------------------------------------------------*/
static void resetSoftware()         // funcdef
{
  BldRc rc;
  if ((rc = bldResetDriver()) != BLD_SUCCESS) {
    logf("Error #%d: %s\n", (int) rc, bldGetErrMsg(rc));
  }
}
#endif


/*---------------------------------------------------- Function Description -*\
  DESCRIPTION
    Resets the Layer 2 and Layer 1 controller/tranceiver for each LI.
\*---------------------------------------------------------------------------*/
static void resetHardware()       // funcdef
{
  BldRc rc;
  int liNo;

  for (liNo = 0; liNo <= LI_MAX; liNo++) {
    if ((rc = bldResetLi(liNo)) != BLD_SUCCESS) {
      // Errors
      logf("Error #%d: %s\n", (int) rc, bldGetErrMsg(rc));
    }
  }
}


/*---------------------------------------------------- Function Description -*\
  DESCRIPTION
\*---------------------------------------------------------------------------*/
static int getBaseAddress()
{
  char choiceStr[256];
  int choice = 0;

  while( choice <= 0 || choice > 13) {

    logf("\nI/O -Base Address Configuration:\n");
    logf("                      Switch (SW1) settings\n");
    logf("Choice    Address     S7 S6 S5 S4 S3 S2 S1\n");
    logf("   1.      0200H       1  1  1  1  1  1  1\n");
    logf("   2.      0220H       1  1  1  1  1  1  0\n");
    logf("   3.      0240H       1  1  1  1  1  0  1\n");
    logf("   4.      0260H       1  1  1  1  1  0  0\n");
    logf("   5.      0280H       1  1  1  1  0  1  1\n");
    logf("   6.      02A0H       1  1  1  1  0  1  0\n");
    logf("   7.      02C0H       1  1  1  1  0  0  1\n");
    logf("   8.      02E0H       1  1  1  1  0  0  0\n");
    logf("   9.      0300H       1  1  1  0  1  1  1\n");
    logf("  10.      0320H       1  1  1  0  1  1  0\n");
    logf("  11.      0340H       1  1  1  0  1  0  1\n");
    logf("  12.      0360H       1  1  1  0  1  0  0\n");
    logf("  13.      0380H       1  1  1  0  0  1  1\n");
    logf("\nGive the I/0-base address setting:\n");
    logf("--> ");
    scanf("%s",choiceStr);
    if ( !stricmp(choiceStr,"q")) {
      exit(0);
    }

    choice = atoi(choiceStr);

  }

  return addressTable[choice];

}

/*---------------------------------------------------- Function Description -*\
  DESCRIPTION
\*---------------------------------------------------------------------------*/
static int getIRQ()
{
  char irqStr[256];
  int irq = 0;
  int goodIRQ = 0;

  while ( !goodIRQ) {

    logf("\nIRQ Configuration:\n");
    logf("IRQ\n");
    logf(" 3\n");
    logf(" 4\n");
    logf(" 5\n");
    logf(" 6\n");
    logf(" 7\n");
    logf(" 9\n");
    logf(" 10\n");
    logf(" 11\n");
    logf(" 12\n");
    logf(" 14\n");
    logf(" 15\n");
    logf("\nGive the IRQ setting:\n");
    logf("--> ");
    scanf("%s", irqStr);
    if ( !stricmp(irqStr,"q")) {
      exit(0);
    }
    irq = atoi(irqStr);

    switch (irq) {
      case 3:
      case 4:
      case 5:
      case 6:
      case 7:
      case 9:
      case 10:
      case 11:
      case 12:
      case 14:
      case 15:
        goodIRQ = 1;
        break;
      default:
        goodIRQ = 0;
        break;
    }

  }

  return irq;

}

static void logf(char *formatStr, ...)
{
  va_list argList;
  va_start(argList, formatStr);

  vfprintf(stdout, formatStr, argList);
  vfprintf(logF,   formatStr, argList);

  va_end(argList);

}


